﻿#Region "FileHeader"
' zoom to current mouse position taken from:
' https://www.codeproject.com/articles/457985/wpf-drawing-canvas-control
' by Kenneth Haugland
#End Region

Public Class DrawingScrollViewer : Inherits ScrollViewer
    Protected Overrides Sub OnScrollChanged(e As ScrollChangedEventArgs)
        MyBase.OnScrollChanged(e)
        'http://social.msdn.microsoft.com/Forums/en-US/wpf/thread/c48379ca-a3e1-4930-a2a1-a7bf5d5f5cf9
        If e.Source Is Me Then
            If e.ExtentHeightChange <> 0 Or e.ExtentWidthChange <> 0 Then
                Dim mousePosition As Point

                mousePosition = Mouse.GetPosition(Me)
                Dim offsetx As Double = e.HorizontalOffset + mousePosition.X
                Dim offsety As Double = e.VerticalOffset + mousePosition.Y

                Dim oldExtentWidth As Double = e.ExtentWidth - e.ExtentWidthChange
                Dim oldExtentHeight As Double = e.ExtentHeight - e.ExtentHeightChange

                Dim relx As Double = offsetx / oldExtentWidth
                Dim rely As Double = offsety / oldExtentHeight

                offsetx = Math.Max(relx * e.ExtentWidth - mousePosition.X, 0)
                offsety = Math.Max(rely * e.ExtentHeight - mousePosition.Y, 0)

                ScrollToHorizontalOffset(offsetx)
                ScrollToVerticalOffset(offsety)
            End If
        End If
    End Sub

    Private _scale As Double = 1
    Private Sub DrawingScrollViewer_PreviewMouseWheel(sender As Object, e As MouseWheelEventArgs) Handles Me.PreviewMouseWheel
        Dim ctrl As Control = TryCast(Content, Control)
        If ctrl Is Nothing Then Exit Sub
        Dim scalefactor As Double = 0
        If e.Delta > 0 Then
            scalefactor = 0.05
        ElseIf e.Delta < 0 Then
            scalefactor = -0.05
        End If
        If _scale + scalefactor < 0.01 Then
            Exit Sub
        End If
        _scale += scalefactor
        Dim scale As ScaleTransform = New ScaleTransform(_scale, _scale)
        ctrl.LayoutTransform = scale
        e.Handled = True
    End Sub

    Private Sub DrawingScrollViewer_Loaded(sender As Object, e As RoutedEventArgs) Handles Me.Loaded
        ScrollToHorizontalOffset(ScrollableWidth / 2)
        ScrollToVerticalOffset(ScrollableHeight / 2)
    End Sub

    Private Sub DrawingScrollViewer_LayoutUpdated(sender As Object, e As EventArgs) Handles Me.LayoutUpdated
        UpdateLayout()
    End Sub
End Class
