﻿#Region "FileHeader"
'    ViewModel is based lossely on
'    http://stackoverflow.com/questions/15579069/graph-nodes-coordinates-evaluation/15580293#15580293
'    https://github.com/High-Core/WPFSamples
'    by Federico Berasategui alias HighCore; originally written in C#
'   
#End Region
Imports System.ComponentModel
Imports System.Collections.ObjectModel

Public Class EditorVM
    Implements INotifyPropertyChanged

#Region "Collections"
    Public ReadOnly Property Nodes As New ObservableCollection(Of Node)
    Public ReadOnly Property Triangles As New ObservableCollection(Of Triangle)
    Public ReadOnly Property Rectangles As New ObservableCollection(Of Rectangle)
#End Region

    Public Sub New()
        ShowAllCoordinates = True
        ShowID = True
    End Sub

    Private _selectedObject As DesignerObject
    Public Property SelectedObject As DesignerObject
        Get
            Return _selectedObject
        End Get
        Set(value As DesignerObject)
            For Each item In Nodes
                item.IsHighlighted = False
            Next
            _selectedObject = value
            OnPropertyChanged("SelectedObject")

        End Set
    End Property

#Region "Creating and Delete DesignerObjects"
    Private _creatingNewNode As Boolean
    Public Property CreatingNewNode As Boolean
        Get
            Return _creatingNewNode
        End Get
        Set(value As Boolean)
            _creatingNewNode = value
            OnPropertyChanged("CreatingNewNode")
            If value Then
                CreateNewNode()
            Else
                Nodes.Where(Function(x) x.IsNew = True).ToList.ForEach(Function(x) Nodes.Remove(x))
            End If
        End Set
    End Property
    Public Sub CreateNewNode()
        Dim newNode As New Node
        Nodes.Add(newNode)
        SelectedObject = newNode
    End Sub



    Private _creatingNewTriangle As Boolean
    Public Property CreatingNewTriangle As Boolean
        Get
            Return _creatingNewTriangle
        End Get
        Set(value As Boolean)
            _creatingNewTriangle = value
            OnPropertyChanged("CreatingNewTriangle")
            If value Then
                CreateNewTriangle()
            Else
                Triangles.Where(Function(x) x.IsNew = True).ToList.ForEach(Function(x) Triangles.Remove(x))
            End If
        End Set
    End Property
    Public Sub CreateNewTriangle()
        Dim newSupport As New Triangle()
        Triangles.Add(newSupport)
        SelectedObject = newSupport
    End Sub


    Private _creatingNewRectangle As Boolean
    Public Property CreatingNewRectangle As Boolean
        Get
            Return _creatingNewRectangle
        End Get
        Set(value As Boolean)
            _creatingNewRectangle = value
            OnPropertyChanged("CreatingNewRectangle")
            If value Then
                CreateNewRectangle()
            Else
                Rectangles.Where(Function(x) x.IsNew = True).ToList.ForEach(Function(x) Rectangles.Remove(x))
            End If
        End Set
    End Property
    Public Sub CreateNewRectangle()
        Dim newOptimRange As New Rectangle
        Rectangles.Add(newOptimRange)
        SelectedObject = newOptimRange
    End Sub


    Public Sub Delete()
        If TypeOf SelectedObject Is Triangle Then
            Dim sup As Triangle = TryCast(SelectedObject, Triangle)
            Triangles.Remove(sup)

        ElseIf TypeOf SelectedObject Is Rectangle Then
            Dim opra As Rectangle = TryCast(SelectedObject, Rectangle)
            Rectangles.Remove(opra)

        ElseIf TypeOf SelectedObject Is Node Then
            Dim node As Node = TryCast(SelectedObject, Node)
            Triangles.Where(Function(x) x.Node.Equals(node)).ToList.ForEach(Function(x) Triangles.Remove(x))
            Rectangles.Where(Function(x) x.Node.Equals(node)).ToList.ForEach(Function(x) Rectangles.Remove(x))
            Nodes.Remove(node)
        End If
        SelectedObject = Nothing
    End Sub
#End Region

#Region "Visibility"
    Private _showID As Boolean
    Public Property ShowID As Boolean
        Get
            Return _showID
        End Get
        Set(value As Boolean)
            _showID = value
            OnPropertyChanged("ShowID")
        End Set
    End Property

    Private _showCurrentCoordinates As Boolean
    Public Property ShowCurrentCoordinates As Boolean
        Get
            Return _showCurrentCoordinates
        End Get
        Set(value As Boolean)
            _showCurrentCoordinates = value
            OnPropertyChanged("ShowCurrentCoordinates")
        End Set
    End Property
    Private _showAllCoordinates As Boolean
    Public Property ShowAllCoordinates As Boolean
        Get
            Return _showAllCoordinates
        End Get
        Set(value As Boolean)
            _showAllCoordinates = value
            OnPropertyChanged("ShowAllCoordinates")
        End Set
    End Property

#End Region


#Region "PropertyChanged"
    Public Event PropertyChanged(sender As Object, e As PropertyChangedEventArgs) Implements INotifyPropertyChanged.PropertyChanged
    Protected Sub OnPropertyChanged(ByVal name As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(name))
    End Sub
#End Region




#Region "Scrolling support"
    Private _areaHeight As Double = 1000
    Public Property AreaHeight As Double
        Get
            Return _areaHeight
        End Get
        Set(value As Double)
            _areaHeight = value
            OnPropertyChanged("AreaHeight")
        End Set
    End Property

    Private _areaWidth As Double = 1000
    Public Property AreaWidth As Double
        Get
            Return _areaWidth
        End Get
        Set(value As Double)
            _areaWidth = value
            OnPropertyChanged("AreaWidth")
        End Set
    End Property
#End Region





End Class

'private Command _deleteCommand;
'public Command DeleteCommand


